@@ -1,5 +1,6 @@
 import json
 from pathlib import Path
+import os
 import pytest
 import re
 import subprocess
@@ -25,12 +26,12 @@ def output_dir(tmp_path):
 
 
 @pytest.fixture
-def check_artfifacts(mocker):
+def check_artifacts(mocker):
     return mocker.patch("tuxmake.build.Build.check_artifacts", return_value=True)
 
 
 @pytest.fixture()
-def Popen(mocker, check_artfifacts):
+def Popen(mocker, check_artifacts):
     _Popen = mocker.patch("subprocess.Popen")
     _Popen.return_value.communicate.return_value = (
         mocker.MagicMock(),
@@ -278,7 +279,7 @@ def test_verbose(linux, mocker, Popen):
 
 def test_default_targets(linux):
     b = Build(tree=linux, targets=[])
-    assert set(t.name for t in b.targets) == set(defaults.targets)
+    assert set(t.name for t in b.targets) == set(defaults.targets) | set(["default"])
 
 
 def test_quiet(linux, capfd):
@@ -436,14 +437,36 @@ class TestModules:
         assert "modules.tar.xz" not in artifacts
 
 
+def tarball_contents(tarball):
+    return subprocess.check_output(["tar", "taf", tarball]).decode("utf-8").splitlines()
+
+
 class TestDtbs:
     def test_dtbs(self, linux):
-        result = build(tree=linux, targets=["dtbs"], target_arch="arm64")
+        result = build(tree=linux, target_arch="arm64")
         artifacts = [str(f.name) for f in result.output_dir.glob("*")]
+        assert result.status["dtbs"].status == "PASS"
+        assert "dtbs/hisilicon/hi6220-hikey.dtb" in tarball_contents(
+            result.output_dir / "dtbs.tar.xz"
+        )
         assert "dtbs.tar.xz" in artifacts
 
+    def test_relative_path_to_source_tree(self, linux):
+        cwd = Path.cwd()
+        try:
+            os.chdir(Path(linux).parent)
+            result = build(tree="linux", target_arch="arm64")
+            assert result.status["dtbs"].status == "PASS"
+            assert "dtbs/hisilicon/hi6220-hikey.dtb" in tarball_contents(
+                result.output_dir / "dtbs.tar.xz"
+            )
+            artifacts = [str(f.name) for f in result.output_dir.glob("*")]
+            assert "dtbs.tar.xz" in artifacts
+        finally:
+            os.chdir(cwd)
+
     def test_skip_on_arch_with_no_dtbs(self, linux):
-        result = build(tree=linux, targets=["dtbs"], target_arch="x86_64")
+        result = build(tree=linux, target_arch="x86_64")
         artifacts = [str(f.name) for f in result.output_dir.glob("*")]
         assert "dtbs.tar.xz" not in artifacts
 
@@ -462,8 +485,12 @@ class TestDtbsLegacy:
         result = build(tree=oldlinux, target_arch="arm64")
         artifacts = [str(f.name) for f in result.output_dir.glob("*")]
         assert "dtbs.tar.xz" in artifacts
+        assert result.status["dtbs-legacy"].status == "PASS"
         errors, _ = result.parse_log()
         assert errors == 0
+        assert "dtbs/hisilicon/hi6220-hikey.dtb" in tarball_contents(
+            result.output_dir / "dtbs.tar.xz"
+        )
 
     def test_collect_dtbs_manually_without_dtbs_install_and_fails(
         self, oldlinux, monkeypatch
@@ -490,7 +517,6 @@ class TestTargetDependencies:
     def test_recursive_dependencies(self, linux):
         result = build(tree=linux, targets=["modules"])
         assert result.status["config"].passed
-        assert result.status["debugkernel"].passed
         assert result.status["modules"].passed
 
 
@@ -703,3 +729,49 @@ class TestMissingArtifacts:
         assert build.failed
         errors, _ = build.parse_log()
         assert errors == 0
+
+    def test_dont_bother_checking_artifacts_if_build_fails(
+        self, linux, check_artifacts, monkeypatch
+    ):
+        monkeypatch.setenv("FAIL", "defconfig")
+        build = Build(tree=linux, targets=["config"])
+        build.run()
+        check_artifacts.assert_not_called()
+
+
+class TestKernel:
+    def test_custom_kernel_image(self, linux):
+        build = Build(
+            tree=linux,
+            target_arch="arm64",
+            targets=["kernel"],
+            kernel_image="Image.bz2",
+        )
+        build.run()
+        assert build.passed
+        assert "Image.bz2" in build.artifacts["kernel"]
+
+    def test_vmlinux(self, linux):
+        build = Build(
+            tree=linux, target_arch="arm64", targets=["kernel"], kernel_image="vmlinux"
+        )
+        build.run()
+        assert build.passed
+        assert "vmlinux" in build.artifacts["kernel"]
+
+
+class TestKselftest:
+    def test_kselftest_merge_before_kselftest(self, linux):
+        build = Build(tree=linux, targets=["kselftest", "kselftest-merge"])
+        names = [t.name for t in build.targets][-2:]
+        assert names == ["kselftest-merge", "kselftest"]
+
+    def test_kselftest_merge_before_kselftest_with_input_already_ordered(self, linux):
+        build = Build(tree=linux, targets=["kselftest-merge", "kselftest"])
+        names = [t.name for t in build.targets][-2:]
+        assert names == ["kselftest-merge", "kselftest"]
+
+    def test_kselftest_without_kselftest_merge(self, linux):
+        build = Build(tree=linux, targets=["kselftest"])
+        names = [t.name for t in build.targets]
+        assert names == ["config", "kselftest"]
