@@ -14,39 +14,51 @@ def builds(home):
 
 
 @pytest.fixture(autouse=True)
-def builder(mocker):
-    b = mocker.patch("tuxmake.cli.build")
+def builder(mocker, linux, tmp_path):
+    b = mocker.patch("tuxmake.cli.Build")
     b.return_value.passed = True
     b.return_value.failed = False
+    b.return_value.source_tree = linux
+    b.return_value.output_dir = tmp_path
     return b
 
 
+@pytest.fixture
+def build(builder):
+    return builder.return_value
+
+
 def args(called):
     return argparse.Namespace(**called.call_args[1])
 
 
-def test_basic_build(builder, monkeypatch):
-    monkeypatch.setattr(sys, "argv", ["tuxmake"])
-    tuxmake()
-    assert args(builder).tree == "."
+class TestBuildRun:
+    def test_basic_build(self, build, monkeypatch):
+        monkeypatch.setattr(sys, "argv", ["tuxmake"])
+        tuxmake()
+        build.run.assert_called()
 
 
-def test_basic_build_with_directory(linux, builder):
-    tree = str(linux)
-    tuxmake("--directory", tree)
-    assert args(builder).tree == tree
+class TestTree:
+    def test_basic_build(self, builder, monkeypatch):
+        monkeypatch.setattr(sys, "argv", ["tuxmake"])
+        tuxmake()
+        assert args(builder).tree == "."
 
+    def test_basic_build_with_directory(self, linux, builder):
+        tree = str(linux)
+        tuxmake("--directory", tree)
+        assert args(builder).tree == tree
 
-def test_build_from_sys_argv(monkeypatch, builder):
-    monkeypatch.setattr(sys, "argv", ["tuxmake", "--directory=/path/to/linux"])
-    tuxmake()
-    assert args(builder).tree == "/path/to/linux"
+    def test_build_from_sys_argv(self, monkeypatch, builder):
+        monkeypatch.setattr(sys, "argv", ["tuxmake", "--directory=/path/to/linux"])
+        tuxmake()
+        assert args(builder).tree == "/path/to/linux"
 
-
-def test_build_from_sys_argv_default_tree_is_cwd(monkeypatch, builder):
-    monkeypatch.setattr(sys, "argv", ["tuxmake"])
-    tuxmake()
-    assert args(builder).tree == "."
+    def test_build_from_sys_argv_default_tree_is_cwd(self, monkeypatch, builder):
+        monkeypatch.setattr(sys, "argv", ["tuxmake"])
+        tuxmake()
+        assert args(builder).tree == "."
 
 
 class TestTargets:
@@ -120,7 +132,7 @@ class TestRuntime:
 class TestImage:
     @pytest.fixture(autouse=True)
     def environment(self, monkeypatch):
-        env = {}
+        env = dict(os.environ)
         monkeypatch.setattr(os, "environ", env)
         return env
 
@@ -311,3 +323,125 @@ class TestOptionsFromEnvironment:
         monkeypatch.setattr(sys, "argv", ["tuxmake", "--toolchain=clang"])
         tuxmake()
         assert args(builder).toolchain == "clang"
+
+
+class TestConfigurationFiles:
+    @pytest.fixture(autouse=True)
+    def config_dir(self, home):
+        confdir = home / ".config" / "tuxmake"
+        confdir.mkdir(parents=True)
+        return confdir
+
+    def test_basics(self, builder, config_dir):
+        with (config_dir / "ccache").open("w") as f:
+            f.write("--wrapper=ccache\n")
+        tuxmake("@ccache")
+        assert args(builder).wrapper == "ccache"
+
+    def test_default(self, builder, config_dir, monkeypatch):
+        with (config_dir / "default").open("w") as f:
+            f.write("--runtime=podman\n")
+        monkeypatch.setattr(sys, "argv", [])
+        tuxmake()
+        assert args(builder).runtime == "podman"
+
+    def test_escaping(self, builder, config_dir):
+        with (config_dir / "escaping").open("w") as f:
+            f.write("--environment=FOO='bar baz'\n")
+        tuxmake("@escaping")
+        assert args(builder).environment["FOO"] == "bar baz"
+
+    def test_order(self, builder, config_dir):
+        with (config_dir / "before").open("w") as f:
+            f.write("--runtime=podman\n")
+        with (config_dir / "after").open("w") as f:
+            f.write("--kconfig=defconfig\n")
+        tuxmake("@before", "--runtime=null", "--kconfig=tinyconfig", "@after")
+        assert args(builder).runtime == "null"
+        assert args(builder).kconfig == "defconfig"
+
+    def test_comments(self, builder, config_dir):
+        with (config_dir / "comments").open("w") as f:
+            f.write("# this is a comment\n")
+        tuxmake("@comments")
+        assert "targets" not in args(builder)
+
+    def test_missing_config_file(self, capsys):
+        with pytest.raises(SystemExit) as exit:
+            tuxmake("@missing")
+        _, err = capsys.readouterr()
+
+        assert exit.value.code == 1
+        assert "E: missing configuration file: " in err
+
+    def test_absolute_path(self, builder, home):
+        with (home / "file").open("w") as f:
+            f.write("--environment=FOO=BAR\n")
+        tuxmake(f"@{home}/file")
+        assert args(builder).environment["FOO"] == "BAR"
+
+    def test_XDG_CONFIG_HOME(self, builder, home, monkeypatch):
+        custom_config_dir = home / "customconfig"
+        (custom_config_dir / "tuxmake").mkdir(parents=True)
+        with (custom_config_dir / "tuxmake" / "myconfig").open("w") as f:
+            f.write("--environment=CONFIG=custom")
+        monkeypatch.setenv("XDG_CONFIG_HOME", str(custom_config_dir))
+        tuxmake("@myconfig")
+        assert args(builder).environment["CONFIG"] == "custom"
+
+
+class TestKernel:
+    def test_kernel(self, builder):
+        tuxmake("--kernel-image=Image.bz2")
+        assert args(builder).kernel_image == "Image.bz2"
+
+
+@pytest.fixture
+def check_call(mocker):
+    return mocker.patch("subprocess.check_call")
+
+
+class TestHooks:
+    def test_pre_build_hook(self, check_call, linux):
+        tuxmake('--pre-build-hook=echo "hello world"')
+        check_call.assert_called_with(
+            ["sh", "-c", 'echo "hello world"'], cwd=str(linux)
+        )
+
+    def test_failed_pre_build_hook_stops_build(self, build, linux):
+        with pytest.raises(SystemExit):
+            tuxmake("--pre-build-hook=false")
+        build.run.assert_not_called()
+
+    def test_post_build_hook(self, check_call, linux):
+        tuxmake("--post-build-hook=git clean -dxf")
+        check_call.assert_called_with(["sh", "-c", "git clean -dxf"], cwd=str(linux))
+
+    def test_post_build_hook_failure_means_exit_failure(self):
+        with pytest.raises(SystemExit) as exit:
+            tuxmake("--post-build-hook=false")
+        assert exit.value.code == 2
+
+    def test_post_build__hook_does_not_run_if_build_failed(self, build, check_call):
+        build.passed = False
+        build.failed = True
+        with pytest.raises(SystemExit) as exit:
+            tuxmake("--post-build-hook=true")
+        assert exit.value.code == 2
+        check_call.assert_not_called()
+
+    def test_results_hook(self, check_call, tmp_path):
+        tuxmake("--results-hook=ls -1")
+        check_call.assert_called_with(["sh", "-c", "ls -1"], cwd=str(tmp_path))
+
+    def test_results_hook_failure_means_exit_failure(self):
+        with pytest.raises(SystemExit) as exit:
+            tuxmake("--results-hook=false")
+        assert exit.value.code == 2
+
+    def test_results_hook_does_not_run_if_build_failed(self, build, check_call):
+        build.passed = False
+        build.failed = True
+        with pytest.raises(SystemExit):
+            tuxmake("--results-hook=true")
+        check_call.assert_not_called()
