@@ -10,7 +10,7 @@ import time
 from tuxmake import __version__
 from tuxmake.arch import Architecture, host_arch
 from tuxmake.toolchain import Toolchain, NoExplicitToolchain
-from tuxmake.wrapper import Wrapper, NoWrapper
+from tuxmake.wrapper import Wrapper
 from tuxmake.output import get_new_output_dir
 from tuxmake.target import create_target
 from tuxmake.runtime import get_runtime
@@ -116,6 +116,8 @@ class Build:
       etc).
     - **targets**: targets to build, list of `str`. If `None` or an empty list
       is passed, the default list of targets will be built.
+    - **kernel_image**: which kernel image to build, overriding the default
+      kernel image name defined for the target architecture.
     - **jobs**: number of concurrent jobs to run (as in `make -j N`). `int`,
       defaults to twice the number of available CPU cores.
     - **runtime:** name of the runtime to use (`str`).
@@ -153,6 +155,7 @@ class Build:
         kconfig_add=[],
         make_variables={},
         targets=defaults.targets,
+        kernel_image=None,
         jobs=None,
         runtime=None,
         verbose=False,
@@ -160,7 +163,7 @@ class Build:
         debug=False,
         auto_cleanup=True,
     ):
-        self.source_tree = tree
+        self.source_tree = Path(tree).absolute()
 
         self.__output_dir__ = None
         self.__output_dir_input__ = output_dir
@@ -173,7 +176,7 @@ class Build:
 
         self.target_arch = target_arch and Architecture(target_arch) or host_arch
         self.toolchain = toolchain and Toolchain(toolchain) or NoExplicitToolchain()
-        self.wrapper = wrapper and Wrapper(wrapper) or NoWrapper()
+        self.wrapper = wrapper and Wrapper(wrapper) or Wrapper("none")
 
         self.environment = environment
 
@@ -188,9 +191,16 @@ class Build:
         if not targets:
             targets = defaults.targets
 
+        if kernel_image:
+            self.target_overrides = {"kernel": kernel_image}
+        else:
+            self.target_overrides = self.target_arch.targets
+
         self.targets = []
+        self.__ordering_only_targets__ = {}
         for t in targets:
             self.add_target(t)
+        self.cleanup_targets()
 
         if jobs:
             self.jobs = jobs
@@ -225,13 +235,27 @@ class Build:
         """
         return self.__status__
 
-    def add_target(self, target_name):
+    def add_target(self, target_name, ordering_only=False):
         target = create_target(target_name, self)
+
+        if ordering_only:
+            if target_name not in self.__ordering_only_targets__:
+                self.__ordering_only_targets__[target_name] = True
+        else:
+            self.__ordering_only_targets__[target_name] = False
+
         for d in target.dependencies:
-            self.add_target(d)
+            self.add_target(d, ordering_only=ordering_only)
+        for a in target.runs_after:
+            self.add_target(a, ordering_only=True)
         if target not in self.targets:
             self.targets.append(target)
 
+    def cleanup_targets(self):
+        self.targets = [
+            t for t in self.targets if not self.__ordering_only_targets__[t.name]
+        ]
+
     def validate(self):
         source = Path(self.source_tree)
         files = [str(f.name) for f in source.glob("*")]
@@ -266,7 +290,7 @@ class Build:
 
         if self.__build_dir_input__:
             self.__build_dir__ = Path(self.__build_dir_input__)
-            self.__build_dir__.mkdir(exist_ok=True)
+            self.__build_dir__.mkdir(parents=True, exist_ok=True)
         else:
             self.__build_dir__ = self.output_dir / "tmp"
             self.__build_dir__.mkdir()
@@ -361,7 +385,7 @@ class Build:
             toolchain=self.toolchain.name,
             wrapper=self.wrapper.name,
             kconfig=self.kconfig,
-            **self.target_arch.targets,
+            **self.target_overrides,
         )
 
     @property
@@ -426,7 +450,7 @@ class Build:
                 fail = True
                 break
 
-        if not self.check_artifacts(target):
+        if not fail and not self.check_artifacts(target):
             fail = True
 
         if fail:
@@ -507,7 +531,7 @@ class Build:
 
     def save_metadata(self):
         with (self.output_dir / "metadata.json").open("w") as f:
-            f.write(json.dumps(self.metadata, indent=4))
+            f.write(json.dumps(self.metadata, indent=4, sort_keys=True))
             f.write("\n")
 
     def parse_log(self):
