@@ -1,19 +1,57 @@
 from datetime import timedelta
 import os
+import subprocess
 import shlex
 import sys
+from tuxmake import xdg
 from tuxmake.arch import Architecture
 from tuxmake.toolchain import Toolchain
-from tuxmake.build import build
+from tuxmake.build import Build
 from tuxmake.cmdline import build_parser
 from tuxmake.exceptions import TuxMakeException
 from tuxmake.runtime import get_runtime
 from tuxmake.utils import supported
 
 
-def main(*argv):
-    if not argv:
-        argv = tuple(sys.argv[1:])
+def read_config(filename, missing_ok=False):
+    configdir = xdg.config_home() / "tuxmake"
+    path = configdir / filename
+    if not path.exists():
+        if missing_ok:
+            return []
+        else:
+            sys.stderr.write(f"E: missing configuration file: {path}\n")
+            sys.exit(1)
+
+    res = []
+    for line in (configdir / filename).read_text().splitlines():
+        if not line.startswith("#"):
+            res += shlex.split(line)
+    return res
+
+
+def run_hooks(hooks, cwd):
+    if not hooks:
+        return
+    for hook in hooks:
+        try:
+            print(hook)
+            subprocess.check_call(["sh", "-c", hook], cwd=str(cwd))
+        except subprocess.CalledProcessError as e:
+            sys.stderr.write(f"E: hook `{hook}` failed with exit code {e.returncode}\n")
+            sys.exit(2)
+
+
+def main(*origargv):
+    if not origargv:
+        origargv = tuple(sys.argv[1:])
+    argv = read_config("default", missing_ok=True)
+    for a in origargv:
+        if a.startswith("@"):
+            argv += read_config(a[1:])
+        else:
+            argv.append(a)
+    argv = tuple(argv)
 
     env_options = os.getenv("TUXMAKE")
     if env_options:
@@ -101,19 +139,34 @@ def main(*argv):
     build_args = {
         k: v
         for k, v in options.__dict__.items()
-        if v and k not in ["color", "docker_image", "image", "shell"]
+        if v
+        and k
+        not in [
+            "color",
+            "docker_image",
+            "image",
+            "shell",
+            "before_hooks",
+            "after_hooks",
+            "results_hooks",
+        ]
     }
     try:
-        result = build(**build_args, auto_cleanup=(not options.shell))
+        build = Build(**build_args, auto_cleanup=(not options.shell))
+        run_hooks(options.before_hooks, build.source_tree)
+        build.run()
         if options.shell:
-            result.run_cmd(["bash"], interactive=True)
-            result.cleanup()
-        for target, info in result.status.items():
+            build.run_cmd(["bash"], interactive=True)
+            build.cleanup()
+        for target, info in build.status.items():
             duration = timedelta(seconds=info.duration)
             print(f"I: {target}: {info.status} in {duration}", file=err)
-        print(f"I: build output in {result.output_dir}", file=err)
-        if result.failed:
+        print(f"I: build output in {build.output_dir}", file=err)
+        if build.failed:
             sys.exit(2)
+        else:
+            run_hooks(options.after_hooks, build.source_tree)
+            run_hooks(options.results_hooks, build.output_dir)
     except TuxMakeException as e:
         sys.stderr.write("E: " + str(e) + "\n")
         sys.exit(1)
